@@ -12,6 +12,7 @@ use console::Term;
 use dialoguer::{theme, Confirm, Input};
 use itertools::Itertools;
 use serde::{de, Deserialize, Serialize};
+use serde_json as json;
 use structopt::clap;
 use structopt::StructOpt;
 
@@ -23,10 +24,12 @@ use crate::v0;
 use crate::Location;
 use crate::Output;
 
+use azure::AzureCredentials;
 use config::Config;
 use helper::{AddLocation, RegisterCluster};
 use print::{ClusterAndStates, Print, StateAndClusters};
 
+mod azure;
 mod config;
 mod helper;
 mod print;
@@ -159,7 +162,7 @@ enum Command {
         cleanup_grace: String,
     },
 
-    #[structopt(about = "Unregister existing cluster", aliases = &["unregister-cl", "uc"], display_order(11))]
+    #[structopt(about = "Unregister existing cluster", aliases = &["unregister-cl", "urc", "uc"], display_order(11))]
     UnregisterCluster {
         #[structopt(help = "Skip confirmation", long, short)]
         force: bool,
@@ -167,16 +170,30 @@ enum Command {
         name: v0::ClusterName,
     },
 
-    #[structopt(about = "List registered clusters", aliases = &["list-cluster", "list-cl", "lc"], display_order(12))]
+    #[structopt(about = "Update registered cluster", aliases = &["update-cl", "upc"], display_order(12))]
+    UpdateCluster {
+        #[structopt(help = "Cluster name")]
+        name: Option<v0::ClusterName>,
+        #[structopt(
+            help = "Namespace where statehub components are installed",
+            long,
+            default_value = "statehub-system"
+        )]
+        namespace: String,
+        #[structopt(flatten)]
+        azure: AzureCredentials,
+    },
+
+    #[structopt(about = "List registered clusters", aliases = &["list-cluster", "list-cl", "lc"], display_order(13))]
     ListClusters,
 
-    #[structopt(about = "Show registered cluster details", aliases = &["show-cl", "sc"], display_order(13))]
+    #[structopt(about = "Show registered cluster details", aliases = &["show-cl", "sc"], display_order(14))]
     ShowCluster {
         #[structopt(help = "Cluster name")]
         name: Option<v0::ClusterName>,
     },
 
-    #[structopt(about = "Diagnose registered cluster for any irregularities", aliases = &["diagnose-cl", "dc"], display_order(14))]
+    #[structopt(about = "Diagnose registered cluster for any irregularities", aliases = &["diagnose-cl", "dc"], display_order(15))]
     DiagnoseCluster {
         #[structopt(help = "Cluster name")]
         name: Option<v0::ClusterName>,
@@ -494,6 +511,22 @@ impl Cli {
             Command::UnregisterCluster { force, name } => {
                 statehub.unregister_cluster(name, force).await
             }
+            Command::UpdateCluster {
+                name,
+                namespace,
+                azure,
+            } => {
+                let name = name.or_else(k8s::get_current_cluster_name).ok_or_else(|| {
+                    anyhow::anyhow!(
+                        "No default Kubernetes context found, need to provide cluster name"
+                    )
+                })?;
+                let kubectl = k8s::Kubectl::new().await?;
+
+                statehub
+                    .update_cluster(kubectl, name, namespace, azure)
+                    .await
+            }
             Command::ListClusters => statehub.list_clusters().await,
             Command::ShowCluster { name } => {
                 let name = name.or_else(k8s::get_current_cluster_name).ok_or_else(|| {
@@ -759,6 +792,23 @@ impl StateHub {
         Ok(())
     }
 
+    async fn update_cluster(
+        &self,
+        kubectl: k8s::Kubectl,
+        name: v0::ClusterName,
+        namespace: String,
+        azure: AzureCredentials,
+    ) -> anyhow::Result<()> {
+        self.inform(format_args!(
+            "Updating cluster with K8s context {} with {:?}",
+            name, azure
+        ))?;
+
+        let azure = json::to_value(azure)?;
+        let _secret = kubectl.store_azure_credentials(namespace, azure).await?;
+        Ok(())
+    }
+
     async fn show_cluster(&self, name: v0::ClusterName) -> anyhow::Result<()> {
         let cluster = self.api.get_cluster(&name).await.map(Detailed)?;
         if let Ok(states) = self.api.get_all_states().await {
