@@ -18,6 +18,7 @@ mod gitlab;
 #[derive(Clone, Debug)]
 pub(super) struct RegisterCluster {
     helm: k8s::Helm,
+    provider: Option<v0::Provider>,
     namespace: String,
     states: Option<Vec<v0::StateName>>,
     default_state: Option<String>,
@@ -28,6 +29,7 @@ pub(super) struct RegisterCluster {
 impl RegisterCluster {
     pub(super) fn new(
         helm: k8s::Helm,
+        provider: Option<v0::Provider>,
         namespace: String,
         states: Option<Vec<v0::StateName>>,
         default_state: Option<String>,
@@ -36,6 +38,7 @@ impl RegisterCluster {
     ) -> Self {
         Self {
             helm,
+            provider,
             namespace,
             states,
             default_state,
@@ -53,14 +56,12 @@ impl StateHub {
     pub(super) async fn register_cluster_helper(
         &self,
         kubectl: k8s::Kubectl,
-        provider: Option<v0::Provider>,
         locations: &[Location],
         register_cluster: RegisterCluster,
     ) -> anyhow::Result<()> {
-        let provider = if let Some(provider) = provider {
-            provider
-        } else {
-            kubectl.get_cluster_provider().await?
+        let provider = match register_cluster.provider {
+            Some(provider) => provider,
+            None => kubectl.get_cluster_provider().await?,
         };
 
         let cluster_name = kubectl.cluster_name();
